"""make lookup table

Revision ID: f0771214ae1e
Revises: 3c5d39510935
Create Date: 2019-01-10 22:09:52.192410

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'f0771214ae1e'
down_revision = '3c5d39510935'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('lookup_table',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('lookup_name', sa.Text(), nullable=False),
    sa.Column('lookup_key', sa.Text(), nullable=True),
    sa.Column('data', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('lookup_table')
    # ### end Alembic commands ###
