"""Add dashbord table

Revision ID: d9fb04ce0ad8
Revises: 7ec59cb49f09
Create Date: 2019-01-28 22:19:28.833566

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'd9fb04ce0ad8'
down_revision = '7ec59cb49f09'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('dashboard',
    sa.Column('collection', sa.Text(), nullable=False),
    sa.UniqueConstraint('collection')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('dashboard')
    # ### end Alembic commands ###
